""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/security_policy
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SecurityPolicySrcintfItem(TypedDict, total=False):
    """Nested item for srcintf field."""
    name: str


class SecurityPolicyDstintfItem(TypedDict, total=False):
    """Nested item for dstintf field."""
    name: str


class SecurityPolicySrcaddrItem(TypedDict, total=False):
    """Nested item for srcaddr field."""
    name: str


class SecurityPolicyDstaddrItem(TypedDict, total=False):
    """Nested item for dstaddr field."""
    name: str


class SecurityPolicySrcaddr6Item(TypedDict, total=False):
    """Nested item for srcaddr6 field."""
    name: str


class SecurityPolicyDstaddr6Item(TypedDict, total=False):
    """Nested item for dstaddr6 field."""
    name: str


class SecurityPolicyInternetservicenameItem(TypedDict, total=False):
    """Nested item for internet-service-name field."""
    name: str


class SecurityPolicyInternetservicegroupItem(TypedDict, total=False):
    """Nested item for internet-service-group field."""
    name: str


class SecurityPolicyInternetservicecustomItem(TypedDict, total=False):
    """Nested item for internet-service-custom field."""
    name: str


class SecurityPolicyInternetservicecustomgroupItem(TypedDict, total=False):
    """Nested item for internet-service-custom-group field."""
    name: str


class SecurityPolicyInternetservicefortiguardItem(TypedDict, total=False):
    """Nested item for internet-service-fortiguard field."""
    name: str


class SecurityPolicyInternetservicesrcnameItem(TypedDict, total=False):
    """Nested item for internet-service-src-name field."""
    name: str


class SecurityPolicyInternetservicesrcgroupItem(TypedDict, total=False):
    """Nested item for internet-service-src-group field."""
    name: str


class SecurityPolicyInternetservicesrccustomItem(TypedDict, total=False):
    """Nested item for internet-service-src-custom field."""
    name: str


class SecurityPolicyInternetservicesrccustomgroupItem(TypedDict, total=False):
    """Nested item for internet-service-src-custom-group field."""
    name: str


class SecurityPolicyInternetservicesrcfortiguardItem(TypedDict, total=False):
    """Nested item for internet-service-src-fortiguard field."""
    name: str


class SecurityPolicyInternetservice6nameItem(TypedDict, total=False):
    """Nested item for internet-service6-name field."""
    name: str


class SecurityPolicyInternetservice6groupItem(TypedDict, total=False):
    """Nested item for internet-service6-group field."""
    name: str


class SecurityPolicyInternetservice6customItem(TypedDict, total=False):
    """Nested item for internet-service6-custom field."""
    name: str


class SecurityPolicyInternetservice6customgroupItem(TypedDict, total=False):
    """Nested item for internet-service6-custom-group field."""
    name: str


class SecurityPolicyInternetservice6fortiguardItem(TypedDict, total=False):
    """Nested item for internet-service6-fortiguard field."""
    name: str


class SecurityPolicyInternetservice6srcnameItem(TypedDict, total=False):
    """Nested item for internet-service6-src-name field."""
    name: str


class SecurityPolicyInternetservice6srcgroupItem(TypedDict, total=False):
    """Nested item for internet-service6-src-group field."""
    name: str


class SecurityPolicyInternetservice6srccustomItem(TypedDict, total=False):
    """Nested item for internet-service6-src-custom field."""
    name: str


class SecurityPolicyInternetservice6srccustomgroupItem(TypedDict, total=False):
    """Nested item for internet-service6-src-custom-group field."""
    name: str


class SecurityPolicyInternetservice6srcfortiguardItem(TypedDict, total=False):
    """Nested item for internet-service6-src-fortiguard field."""
    name: str


class SecurityPolicyServiceItem(TypedDict, total=False):
    """Nested item for service field."""
    name: str


class SecurityPolicyApplicationItem(TypedDict, total=False):
    """Nested item for application field."""
    id: int


class SecurityPolicyAppcategoryItem(TypedDict, total=False):
    """Nested item for app-category field."""
    id: int


class SecurityPolicyAppgroupItem(TypedDict, total=False):
    """Nested item for app-group field."""
    name: str


class SecurityPolicyGroupsItem(TypedDict, total=False):
    """Nested item for groups field."""
    name: str


class SecurityPolicyUsersItem(TypedDict, total=False):
    """Nested item for users field."""
    name: str


class SecurityPolicyFssogroupsItem(TypedDict, total=False):
    """Nested item for fsso-groups field."""
    name: str


class SecurityPolicyPayload(TypedDict, total=False):
    """Payload type for SecurityPolicy operations."""
    uuid: str
    policyid: int
    name: str
    comments: str
    srcintf: str | list[str] | list[SecurityPolicySrcintfItem]
    dstintf: str | list[str] | list[SecurityPolicyDstintfItem]
    srcaddr: str | list[str] | list[SecurityPolicySrcaddrItem]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: str | list[str] | list[SecurityPolicyDstaddrItem]
    dstaddr_negate: Literal["enable", "disable"]
    srcaddr6: str | list[str] | list[SecurityPolicySrcaddr6Item]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr6: str | list[str] | list[SecurityPolicyDstaddr6Item]
    dstaddr6_negate: Literal["enable", "disable"]
    internet_service: Literal["enable", "disable"]
    internet_service_name: str | list[str] | list[SecurityPolicyInternetservicenameItem]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_group: str | list[str] | list[SecurityPolicyInternetservicegroupItem]
    internet_service_custom: str | list[str] | list[SecurityPolicyInternetservicecustomItem]
    internet_service_custom_group: str | list[str] | list[SecurityPolicyInternetservicecustomgroupItem]
    internet_service_fortiguard: str | list[str] | list[SecurityPolicyInternetservicefortiguardItem]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: str | list[str] | list[SecurityPolicyInternetservicesrcnameItem]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service_src_group: str | list[str] | list[SecurityPolicyInternetservicesrcgroupItem]
    internet_service_src_custom: str | list[str] | list[SecurityPolicyInternetservicesrccustomItem]
    internet_service_src_custom_group: str | list[str] | list[SecurityPolicyInternetservicesrccustomgroupItem]
    internet_service_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservicesrcfortiguardItem]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: str | list[str] | list[SecurityPolicyInternetservice6nameItem]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_group: str | list[str] | list[SecurityPolicyInternetservice6groupItem]
    internet_service6_custom: str | list[str] | list[SecurityPolicyInternetservice6customItem]
    internet_service6_custom_group: str | list[str] | list[SecurityPolicyInternetservice6customgroupItem]
    internet_service6_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6fortiguardItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: str | list[str] | list[SecurityPolicyInternetservice6srcnameItem]
    internet_service6_src_negate: Literal["enable", "disable"]
    internet_service6_src_group: str | list[str] | list[SecurityPolicyInternetservice6srcgroupItem]
    internet_service6_src_custom: str | list[str] | list[SecurityPolicyInternetservice6srccustomItem]
    internet_service6_src_custom_group: str | list[str] | list[SecurityPolicyInternetservice6srccustomgroupItem]
    internet_service6_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6srcfortiguardItem]
    enforce_default_app_port: Literal["enable", "disable"]
    service: str | list[str] | list[SecurityPolicyServiceItem]
    service_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    send_deny_packet: Literal["disable", "enable"]
    schedule: str
    status: Literal["enable", "disable"]
    logtraffic: Literal["all", "utm", "disable"]
    learning_mode: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    ssh_filter_profile: str
    casb_profile: str
    application: str | list[str] | list[SecurityPolicyApplicationItem]
    app_category: str | list[str] | list[SecurityPolicyAppcategoryItem]
    url_category: str | list[str]
    app_group: str | list[str] | list[SecurityPolicyAppgroupItem]
    groups: str | list[str] | list[SecurityPolicyGroupsItem]
    users: str | list[str] | list[SecurityPolicyUsersItem]
    fsso_groups: str | list[str] | list[SecurityPolicyFssogroupsItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SecurityPolicyResponse(TypedDict, total=False):
    """Response type for SecurityPolicy - use with .dict property for typed dict access."""
    uuid: str
    policyid: int
    name: str
    comments: str
    srcintf: list[SecurityPolicySrcintfItem]
    dstintf: list[SecurityPolicyDstintfItem]
    srcaddr: list[SecurityPolicySrcaddrItem]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: list[SecurityPolicyDstaddrItem]
    dstaddr_negate: Literal["enable", "disable"]
    srcaddr6: list[SecurityPolicySrcaddr6Item]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr6: list[SecurityPolicyDstaddr6Item]
    dstaddr6_negate: Literal["enable", "disable"]
    internet_service: Literal["enable", "disable"]
    internet_service_name: list[SecurityPolicyInternetservicenameItem]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_group: list[SecurityPolicyInternetservicegroupItem]
    internet_service_custom: list[SecurityPolicyInternetservicecustomItem]
    internet_service_custom_group: list[SecurityPolicyInternetservicecustomgroupItem]
    internet_service_fortiguard: list[SecurityPolicyInternetservicefortiguardItem]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: list[SecurityPolicyInternetservicesrcnameItem]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service_src_group: list[SecurityPolicyInternetservicesrcgroupItem]
    internet_service_src_custom: list[SecurityPolicyInternetservicesrccustomItem]
    internet_service_src_custom_group: list[SecurityPolicyInternetservicesrccustomgroupItem]
    internet_service_src_fortiguard: list[SecurityPolicyInternetservicesrcfortiguardItem]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: list[SecurityPolicyInternetservice6nameItem]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_group: list[SecurityPolicyInternetservice6groupItem]
    internet_service6_custom: list[SecurityPolicyInternetservice6customItem]
    internet_service6_custom_group: list[SecurityPolicyInternetservice6customgroupItem]
    internet_service6_fortiguard: list[SecurityPolicyInternetservice6fortiguardItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: list[SecurityPolicyInternetservice6srcnameItem]
    internet_service6_src_negate: Literal["enable", "disable"]
    internet_service6_src_group: list[SecurityPolicyInternetservice6srcgroupItem]
    internet_service6_src_custom: list[SecurityPolicyInternetservice6srccustomItem]
    internet_service6_src_custom_group: list[SecurityPolicyInternetservice6srccustomgroupItem]
    internet_service6_src_fortiguard: list[SecurityPolicyInternetservice6srcfortiguardItem]
    enforce_default_app_port: Literal["enable", "disable"]
    service: list[SecurityPolicyServiceItem]
    service_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    send_deny_packet: Literal["disable", "enable"]
    schedule: str
    status: Literal["enable", "disable"]
    logtraffic: Literal["all", "utm", "disable"]
    learning_mode: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    ssh_filter_profile: str
    casb_profile: str
    application: list[SecurityPolicyApplicationItem]
    app_category: list[SecurityPolicyAppcategoryItem]
    url_category: str | list[str]
    app_group: list[SecurityPolicyAppgroupItem]
    groups: list[SecurityPolicyGroupsItem]
    users: list[SecurityPolicyUsersItem]
    fsso_groups: list[SecurityPolicyFssogroupsItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SecurityPolicySrcintfItemObject(FortiObject[SecurityPolicySrcintfItem]):
    """Typed object for srcintf table items with attribute access."""
    name: str


class SecurityPolicyDstintfItemObject(FortiObject[SecurityPolicyDstintfItem]):
    """Typed object for dstintf table items with attribute access."""
    name: str


class SecurityPolicySrcaddrItemObject(FortiObject[SecurityPolicySrcaddrItem]):
    """Typed object for srcaddr table items with attribute access."""
    name: str


class SecurityPolicyDstaddrItemObject(FortiObject[SecurityPolicyDstaddrItem]):
    """Typed object for dstaddr table items with attribute access."""
    name: str


class SecurityPolicySrcaddr6ItemObject(FortiObject[SecurityPolicySrcaddr6Item]):
    """Typed object for srcaddr6 table items with attribute access."""
    name: str


class SecurityPolicyDstaddr6ItemObject(FortiObject[SecurityPolicyDstaddr6Item]):
    """Typed object for dstaddr6 table items with attribute access."""
    name: str


class SecurityPolicyInternetservicenameItemObject(FortiObject[SecurityPolicyInternetservicenameItem]):
    """Typed object for internet-service-name table items with attribute access."""
    name: str


class SecurityPolicyInternetservicegroupItemObject(FortiObject[SecurityPolicyInternetservicegroupItem]):
    """Typed object for internet-service-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservicecustomItemObject(FortiObject[SecurityPolicyInternetservicecustomItem]):
    """Typed object for internet-service-custom table items with attribute access."""
    name: str


class SecurityPolicyInternetservicecustomgroupItemObject(FortiObject[SecurityPolicyInternetservicecustomgroupItem]):
    """Typed object for internet-service-custom-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservicefortiguardItemObject(FortiObject[SecurityPolicyInternetservicefortiguardItem]):
    """Typed object for internet-service-fortiguard table items with attribute access."""
    name: str


class SecurityPolicyInternetservicesrcnameItemObject(FortiObject[SecurityPolicyInternetservicesrcnameItem]):
    """Typed object for internet-service-src-name table items with attribute access."""
    name: str


class SecurityPolicyInternetservicesrcgroupItemObject(FortiObject[SecurityPolicyInternetservicesrcgroupItem]):
    """Typed object for internet-service-src-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservicesrccustomItemObject(FortiObject[SecurityPolicyInternetservicesrccustomItem]):
    """Typed object for internet-service-src-custom table items with attribute access."""
    name: str


class SecurityPolicyInternetservicesrccustomgroupItemObject(FortiObject[SecurityPolicyInternetservicesrccustomgroupItem]):
    """Typed object for internet-service-src-custom-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservicesrcfortiguardItemObject(FortiObject[SecurityPolicyInternetservicesrcfortiguardItem]):
    """Typed object for internet-service-src-fortiguard table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6nameItemObject(FortiObject[SecurityPolicyInternetservice6nameItem]):
    """Typed object for internet-service6-name table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6groupItemObject(FortiObject[SecurityPolicyInternetservice6groupItem]):
    """Typed object for internet-service6-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6customItemObject(FortiObject[SecurityPolicyInternetservice6customItem]):
    """Typed object for internet-service6-custom table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6customgroupItemObject(FortiObject[SecurityPolicyInternetservice6customgroupItem]):
    """Typed object for internet-service6-custom-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6fortiguardItemObject(FortiObject[SecurityPolicyInternetservice6fortiguardItem]):
    """Typed object for internet-service6-fortiguard table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6srcnameItemObject(FortiObject[SecurityPolicyInternetservice6srcnameItem]):
    """Typed object for internet-service6-src-name table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6srcgroupItemObject(FortiObject[SecurityPolicyInternetservice6srcgroupItem]):
    """Typed object for internet-service6-src-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6srccustomItemObject(FortiObject[SecurityPolicyInternetservice6srccustomItem]):
    """Typed object for internet-service6-src-custom table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6srccustomgroupItemObject(FortiObject[SecurityPolicyInternetservice6srccustomgroupItem]):
    """Typed object for internet-service6-src-custom-group table items with attribute access."""
    name: str


class SecurityPolicyInternetservice6srcfortiguardItemObject(FortiObject[SecurityPolicyInternetservice6srcfortiguardItem]):
    """Typed object for internet-service6-src-fortiguard table items with attribute access."""
    name: str


class SecurityPolicyServiceItemObject(FortiObject[SecurityPolicyServiceItem]):
    """Typed object for service table items with attribute access."""
    name: str


class SecurityPolicyApplicationItemObject(FortiObject[SecurityPolicyApplicationItem]):
    """Typed object for application table items with attribute access."""
    id: int


class SecurityPolicyAppcategoryItemObject(FortiObject[SecurityPolicyAppcategoryItem]):
    """Typed object for app-category table items with attribute access."""
    id: int


class SecurityPolicyAppgroupItemObject(FortiObject[SecurityPolicyAppgroupItem]):
    """Typed object for app-group table items with attribute access."""
    name: str


class SecurityPolicyGroupsItemObject(FortiObject[SecurityPolicyGroupsItem]):
    """Typed object for groups table items with attribute access."""
    name: str


class SecurityPolicyUsersItemObject(FortiObject[SecurityPolicyUsersItem]):
    """Typed object for users table items with attribute access."""
    name: str


class SecurityPolicyFssogroupsItemObject(FortiObject[SecurityPolicyFssogroupsItem]):
    """Typed object for fsso-groups table items with attribute access."""
    name: str


class SecurityPolicyObject(FortiObject):
    """Typed FortiObject for SecurityPolicy with field access."""
    uuid: str
    policyid: int
    name: str
    comments: str
    srcintf: FortiObjectList[SecurityPolicySrcintfItemObject]
    dstintf: FortiObjectList[SecurityPolicyDstintfItemObject]
    srcaddr: FortiObjectList[SecurityPolicySrcaddrItemObject]
    srcaddr_negate: Literal["enable", "disable"]
    dstaddr: FortiObjectList[SecurityPolicyDstaddrItemObject]
    dstaddr_negate: Literal["enable", "disable"]
    srcaddr6: FortiObjectList[SecurityPolicySrcaddr6ItemObject]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr6: FortiObjectList[SecurityPolicyDstaddr6ItemObject]
    dstaddr6_negate: Literal["enable", "disable"]
    internet_service: Literal["enable", "disable"]
    internet_service_name: FortiObjectList[SecurityPolicyInternetservicenameItemObject]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_group: FortiObjectList[SecurityPolicyInternetservicegroupItemObject]
    internet_service_custom: FortiObjectList[SecurityPolicyInternetservicecustomItemObject]
    internet_service_custom_group: FortiObjectList[SecurityPolicyInternetservicecustomgroupItemObject]
    internet_service_fortiguard: FortiObjectList[SecurityPolicyInternetservicefortiguardItemObject]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: FortiObjectList[SecurityPolicyInternetservicesrcnameItemObject]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service_src_group: FortiObjectList[SecurityPolicyInternetservicesrcgroupItemObject]
    internet_service_src_custom: FortiObjectList[SecurityPolicyInternetservicesrccustomItemObject]
    internet_service_src_custom_group: FortiObjectList[SecurityPolicyInternetservicesrccustomgroupItemObject]
    internet_service_src_fortiguard: FortiObjectList[SecurityPolicyInternetservicesrcfortiguardItemObject]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: FortiObjectList[SecurityPolicyInternetservice6nameItemObject]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_group: FortiObjectList[SecurityPolicyInternetservice6groupItemObject]
    internet_service6_custom: FortiObjectList[SecurityPolicyInternetservice6customItemObject]
    internet_service6_custom_group: FortiObjectList[SecurityPolicyInternetservice6customgroupItemObject]
    internet_service6_fortiguard: FortiObjectList[SecurityPolicyInternetservice6fortiguardItemObject]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: FortiObjectList[SecurityPolicyInternetservice6srcnameItemObject]
    internet_service6_src_negate: Literal["enable", "disable"]
    internet_service6_src_group: FortiObjectList[SecurityPolicyInternetservice6srcgroupItemObject]
    internet_service6_src_custom: FortiObjectList[SecurityPolicyInternetservice6srccustomItemObject]
    internet_service6_src_custom_group: FortiObjectList[SecurityPolicyInternetservice6srccustomgroupItemObject]
    internet_service6_src_fortiguard: FortiObjectList[SecurityPolicyInternetservice6srcfortiguardItemObject]
    enforce_default_app_port: Literal["enable", "disable"]
    service: FortiObjectList[SecurityPolicyServiceItemObject]
    service_negate: Literal["enable", "disable"]
    action: Literal["accept", "deny"]
    send_deny_packet: Literal["disable", "enable"]
    schedule: str
    status: Literal["enable", "disable"]
    logtraffic: Literal["all", "utm", "disable"]
    learning_mode: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    ssh_filter_profile: str
    casb_profile: str
    application: FortiObjectList[SecurityPolicyApplicationItemObject]
    app_category: FortiObjectList[SecurityPolicyAppcategoryItemObject]
    url_category: str | list[str]
    app_group: FortiObjectList[SecurityPolicyAppgroupItemObject]
    groups: FortiObjectList[SecurityPolicyGroupsItemObject]
    users: FortiObjectList[SecurityPolicyUsersItemObject]
    fsso_groups: FortiObjectList[SecurityPolicyFssogroupsItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class SecurityPolicy:
    """
    
    Endpoint: firewall/security_policy
    Category: cmdb
    MKey: policyid
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        policyid: int,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SecurityPolicyObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[SecurityPolicyObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: SecurityPolicyPayload | None = ...,
        uuid: str | None = ...,
        policyid: int | None = ...,
        name: str | None = ...,
        comments: str | None = ...,
        srcintf: str | list[str] | list[SecurityPolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[SecurityPolicyDstintfItem] | None = ...,
        srcaddr: str | list[str] | list[SecurityPolicySrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[SecurityPolicyDstaddrItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6: str | list[str] | list[SecurityPolicySrcaddr6Item] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6: str | list[str] | list[SecurityPolicyDstaddr6Item] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[SecurityPolicyInternetservicenameItem] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_group: str | list[str] | list[SecurityPolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[SecurityPolicyInternetservicecustomItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[SecurityPolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[SecurityPolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[SecurityPolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_group: str | list[str] | list[SecurityPolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[SecurityPolicyInternetservicesrccustomItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[SecurityPolicyInternetservicesrccustomgroupItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[SecurityPolicyInternetservice6nameItem] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_group: str | list[str] | list[SecurityPolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[SecurityPolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[SecurityPolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[SecurityPolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_group: str | list[str] | list[SecurityPolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[SecurityPolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[SecurityPolicyInternetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6srcfortiguardItem] | None = ...,
        enforce_default_app_port: Literal["enable", "disable"] | None = ...,
        service: str | list[str] | list[SecurityPolicyServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        learning_mode: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        application: str | list[str] | list[SecurityPolicyApplicationItem] | None = ...,
        app_category: str | list[str] | list[SecurityPolicyAppcategoryItem] | None = ...,
        url_category: str | list[str] | None = ...,
        app_group: str | list[str] | list[SecurityPolicyAppgroupItem] | None = ...,
        groups: str | list[str] | list[SecurityPolicyGroupsItem] | None = ...,
        users: str | list[str] | list[SecurityPolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[SecurityPolicyFssogroupsItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SecurityPolicyObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SecurityPolicyPayload | None = ...,
        uuid: str | None = ...,
        policyid: int | None = ...,
        name: str | None = ...,
        comments: str | None = ...,
        srcintf: str | list[str] | list[SecurityPolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[SecurityPolicyDstintfItem] | None = ...,
        srcaddr: str | list[str] | list[SecurityPolicySrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[SecurityPolicyDstaddrItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6: str | list[str] | list[SecurityPolicySrcaddr6Item] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6: str | list[str] | list[SecurityPolicyDstaddr6Item] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[SecurityPolicyInternetservicenameItem] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_group: str | list[str] | list[SecurityPolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[SecurityPolicyInternetservicecustomItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[SecurityPolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[SecurityPolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[SecurityPolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_group: str | list[str] | list[SecurityPolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[SecurityPolicyInternetservicesrccustomItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[SecurityPolicyInternetservicesrccustomgroupItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[SecurityPolicyInternetservice6nameItem] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_group: str | list[str] | list[SecurityPolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[SecurityPolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[SecurityPolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[SecurityPolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_group: str | list[str] | list[SecurityPolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[SecurityPolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[SecurityPolicyInternetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6srcfortiguardItem] | None = ...,
        enforce_default_app_port: Literal["enable", "disable"] | None = ...,
        service: str | list[str] | list[SecurityPolicyServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        learning_mode: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        application: str | list[str] | list[SecurityPolicyApplicationItem] | None = ...,
        app_category: str | list[str] | list[SecurityPolicyAppcategoryItem] | None = ...,
        url_category: str | list[str] | None = ...,
        app_group: str | list[str] | list[SecurityPolicyAppgroupItem] | None = ...,
        groups: str | list[str] | list[SecurityPolicyGroupsItem] | None = ...,
        users: str | list[str] | list[SecurityPolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[SecurityPolicyFssogroupsItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SecurityPolicyObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SecurityPolicyPayload | None = ...,
        uuid: str | None = ...,
        policyid: int | None = ...,
        name: str | None = ...,
        comments: str | None = ...,
        srcintf: str | list[str] | list[SecurityPolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[SecurityPolicyDstintfItem] | None = ...,
        srcaddr: str | list[str] | list[SecurityPolicySrcaddrItem] | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr: str | list[str] | list[SecurityPolicyDstaddrItem] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6: str | list[str] | list[SecurityPolicySrcaddr6Item] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6: str | list[str] | list[SecurityPolicyDstaddr6Item] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[SecurityPolicyInternetservicenameItem] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_group: str | list[str] | list[SecurityPolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[SecurityPolicyInternetservicecustomItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[SecurityPolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[SecurityPolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[SecurityPolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_group: str | list[str] | list[SecurityPolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[SecurityPolicyInternetservicesrccustomItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[SecurityPolicyInternetservicesrccustomgroupItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[SecurityPolicyInternetservice6nameItem] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_group: str | list[str] | list[SecurityPolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[SecurityPolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[SecurityPolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[SecurityPolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_group: str | list[str] | list[SecurityPolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[SecurityPolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[SecurityPolicyInternetservice6srccustomgroupItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[SecurityPolicyInternetservice6srcfortiguardItem] | None = ...,
        enforce_default_app_port: Literal["enable", "disable"] | None = ...,
        service: str | list[str] | list[SecurityPolicyServiceItem] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        action: Literal["accept", "deny"] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        schedule: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        learning_mode: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        application: str | list[str] | list[SecurityPolicyApplicationItem] | None = ...,
        app_category: str | list[str] | list[SecurityPolicyAppcategoryItem] | None = ...,
        url_category: str | list[str] | None = ...,
        app_group: str | list[str] | list[SecurityPolicyAppgroupItem] | None = ...,
        groups: str | list[str] | list[SecurityPolicyGroupsItem] | None = ...,
        users: str | list[str] | list[SecurityPolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[SecurityPolicyFssogroupsItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "SecurityPolicy",
    "SecurityPolicyPayload",
    "SecurityPolicyResponse",
    "SecurityPolicyObject",
]


"""FortiOS CMDB - CentralManagement category"""

from .status import Status

__all__ = [
    "CentralManagement",
    "Status",
]


class CentralManagement:
    """CentralManagement endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """CentralManagement endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.status = Status(client)



"""FortiOS CMDB - ProxyPolicy category (stub)"""

from typing import Any
from ..proxy_policy_base import ProxyPolicy as ProxyPolicyBase
from .clear_counters import ClearCounters

class ProxyPolicy(ProxyPolicyBase):
    """ProxyPolicy endpoints wrapper for CMDB API."""

    clear_counters: ClearCounters

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""FortiOS CMDB - RogueAp category"""

from ..rogue_ap_base import RogueAp as RogueApBase
from .clear_all import ClearAll
from .set_status import SetStatus

__all__ = [
    "ClearAll",
    "RogueAp",
    "SetStatus",
]


class RogueAp(RogueApBase):
    """RogueAp endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """RogueAp endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.clear_all = ClearAll(client)
        self.set_status = SetStatus(client)



"""
Pydantic Models for CMDB - log/syslogd/override_setting

Runtime validation models for log/syslogd/override_setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class OverrideSettingCustomFieldName(BaseModel):
    """
    Child table model for custom-field-name.
    
    Custom field name for CEF format logging.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=255, default=0, serialization_alias="id", description="Entry ID.")    
    name: str = Field(max_length=35, description="Field name [A-Za-z0-9_].")    
    custom: str = Field(max_length=35, description="Field custom name [A-Za-z0-9_].")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class OverrideSettingFacilityEnum(str, Enum):
    """Allowed values for facility field."""
    KERNEL = "kernel"
    USER = "user"
    MAIL = "mail"
    DAEMON = "daemon"
    AUTH = "auth"
    SYSLOG = "syslog"
    LPR = "lpr"
    NEWS = "news"
    UUCP = "uucp"
    CRON = "cron"
    AUTHPRIV = "authpriv"
    FTP = "ftp"
    NTP = "ntp"
    AUDIT = "audit"
    ALERT = "alert"
    CLOCK = "clock"
    LOCAL0 = "local0"
    LOCAL1 = "local1"
    LOCAL2 = "local2"
    LOCAL3 = "local3"
    LOCAL4 = "local4"
    LOCAL5 = "local5"
    LOCAL6 = "local6"
    LOCAL7 = "local7"

class OverrideSettingFormatEnum(str, Enum):
    """Allowed values for format_ field."""
    DEFAULT = "default"
    CSV = "csv"
    CEF = "cef"
    RFC5424 = "rfc5424"
    JSON = "json"

class OverrideSettingEncAlgorithmEnum(str, Enum):
    """Allowed values for enc_algorithm field."""
    HIGH_MEDIUM = "high-medium"
    HIGH = "high"
    LOW = "low"
    DISABLE = "disable"

class OverrideSettingSslMinProtoVersionEnum(str, Enum):
    """Allowed values for ssl_min_proto_version field."""
    DEFAULT = "default"
    SSLV3 = "SSLv3"
    TLSV1 = "TLSv1"
    TLSV1_1 = "TLSv1-1"
    TLSV1_2 = "TLSv1-2"
    TLSV1_3 = "TLSv1-3"


# ============================================================================
# Main Model
# ============================================================================

class OverrideSettingModel(BaseModel):
    """
    Pydantic model for log/syslogd/override_setting configuration.
    
    Override settings for remote syslog server.
    
    Validation Rules:        - status: pattern=        - server: max_length=127 pattern=        - mode: pattern=        - use_management_vdom: pattern=        - port: min=0 max=65535 pattern=        - facility: pattern=        - source_ip_interface: max_length=15 pattern=        - source_ip: max_length=63 pattern=        - format_: pattern=        - priority: pattern=        - max_log_rate: min=0 max=100000 pattern=        - enc_algorithm: pattern=        - ssl_min_proto_version: pattern=        - certificate: max_length=35 pattern=        - custom_field_name: pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable remote syslog logging.")    
    server: str = Field(max_length=127, description="Address of remote syslog server.")    
    mode: Literal["udp", "legacy-reliable", "reliable"] | None = Field(default="udp", description="Remote syslog logging over UDP/Reliable TCP.")    
    use_management_vdom: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of management VDOM as source VDOM for logs sent to syslog server.")    
    port: int | None = Field(ge=0, le=65535, default=514, description="Server listen port.")    
    facility: OverrideSettingFacilityEnum | None = Field(default=OverrideSettingFacilityEnum.LOCAL7, description="Remote syslog facility.")    
    source_ip_interface: str | None = Field(max_length=15, default=None, description="Source interface of syslog.")  # datasource: ['system.interface.name']    
    source_ip: str | None = Field(max_length=63, default=None, description="Source IP address of syslog.")    
    format_: OverrideSettingFormatEnum | None = Field(default=OverrideSettingFormatEnum.DEFAULT, serialization_alias="format", description="Log format.")    
    priority: Literal["default", "low"] | None = Field(default="default", description="Set log transmission priority.")    
    max_log_rate: int | None = Field(ge=0, le=100000, default=0, description="Syslog maximum log rate in MBps (0 = unlimited).")    
    enc_algorithm: OverrideSettingEncAlgorithmEnum | None = Field(default=OverrideSettingEncAlgorithmEnum.DISABLE, description="Enable/disable reliable syslogging with TLS encryption.")    
    ssl_min_proto_version: OverrideSettingSslMinProtoVersionEnum | None = Field(default=OverrideSettingSslMinProtoVersionEnum.DEFAULT, description="Minimum supported protocol version for SSL/TLS connections (default is to follow system global setting).")    
    certificate: str | None = Field(max_length=35, default=None, description="Certificate used to communicate with Syslog server.")  # datasource: ['certificate.local.name']    
    custom_field_name: list[OverrideSettingCustomFieldName] = Field(default_factory=list, description="Custom field name for CEF format logging.")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('source_ip_interface')
    @classmethod
    def validate_source_ip_interface(cls, v: Any) -> Any:
        """
        Validate source_ip_interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certificate')
    @classmethod
    def validate_certificate(cls, v: Any) -> Any:
        """
        Validate certificate field.
        
        Datasource: ['certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "OverrideSettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_source_ip_interface_references(self, client: Any) -> list[str]:
        """
        Validate source_ip_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideSettingModel(
            ...     source_ip_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_source_ip_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.log.syslogd.override_setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "source_ip_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Source-Ip-Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_certificate_references(self, client: Any) -> list[str]:
        """
        Validate certificate references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideSettingModel(
            ...     certificate="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certificate_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.log.syslogd.override_setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certificate", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certificate '{value}' not found in "
                "certificate/local"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideSettingModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.log.syslogd.override_setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_source_ip_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certificate_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "OverrideSettingModel",    "OverrideSettingCustomFieldName",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:02.396832Z
# ============================================================================


"""Validation helpers for user/exchange - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "server-name",  # MS Exchange server hostname.
    "domain-name",  # MS Exchange server fully qualified domain name.
    "username",  # User name used to sign in to the server. Must have proper permissions for service.
    "password",  # Password for the specified username.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "server-name": "",
    "domain-name": "",
    "username": "",
    "ip": "0.0.0.0",
    "connect-protocol": "rpc-over-https",
    "validate-server-certificate": "disable",
    "auth-type": "kerberos",
    "auth-level": "privacy",
    "http-auth-type": "ntlm",
    "ssl-min-proto-version": "default",
    "auto-discover-kdc": "enable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # MS Exchange server entry name.
    "server-name": "string",  # MS Exchange server hostname.
    "domain-name": "string",  # MS Exchange server fully qualified domain name.
    "username": "string",  # User name used to sign in to the server. Must have proper pe
    "password": "password",  # Password for the specified username.
    "ip": "ipv4-address-any",  # Server IPv4 address.
    "connect-protocol": "option",  # Connection protocol used to connect to MS Exchange service.
    "validate-server-certificate": "option",  # Enable/disable exchange server certificate validation.
    "auth-type": "option",  # Authentication security type used for the RPC protocol layer
    "auth-level": "option",  # Authentication security level used for the RPC protocol laye
    "http-auth-type": "option",  # Authentication security type used for the HTTP transport.
    "ssl-min-proto-version": "option",  # Minimum SSL/TLS protocol version for HTTPS transport (defaul
    "auto-discover-kdc": "option",  # Enable/disable automatic discovery of KDC IP addresses.
    "kdc-ip": "string",  # KDC IPv4 addresses for Kerberos authentication.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "MS Exchange server entry name.",
    "server-name": "MS Exchange server hostname.",
    "domain-name": "MS Exchange server fully qualified domain name.",
    "username": "User name used to sign in to the server. Must have proper permissions for service.",
    "password": "Password for the specified username.",
    "ip": "Server IPv4 address.",
    "connect-protocol": "Connection protocol used to connect to MS Exchange service.",
    "validate-server-certificate": "Enable/disable exchange server certificate validation.",
    "auth-type": "Authentication security type used for the RPC protocol layer.",
    "auth-level": "Authentication security level used for the RPC protocol layer.",
    "http-auth-type": "Authentication security type used for the HTTP transport.",
    "ssl-min-proto-version": "Minimum SSL/TLS protocol version for HTTPS transport (default is to follow system global setting).",
    "auto-discover-kdc": "Enable/disable automatic discovery of KDC IP addresses.",
    "kdc-ip": "KDC IPv4 addresses for Kerberos authentication.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "server-name": {"type": "string", "max_length": 63},
    "domain-name": {"type": "string", "max_length": 79},
    "username": {"type": "string", "max_length": 64},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "kdc-ip": {
        "ipv4": {
            "type": "string",
            "help": "KDC IPv4 addresses for Kerberos authentication.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_CONNECT_PROTOCOL = [
    "rpc-over-tcp",
    "rpc-over-http",
    "rpc-over-https",
]
VALID_BODY_VALIDATE_SERVER_CERTIFICATE = [
    "disable",
    "enable",
]
VALID_BODY_AUTH_TYPE = [
    "spnego",
    "ntlm",
    "kerberos",
]
VALID_BODY_AUTH_LEVEL = [
    "connect",
    "call",
    "packet",
    "integrity",
    "privacy",
]
VALID_BODY_HTTP_AUTH_TYPE = [
    "basic",
    "ntlm",
]
VALID_BODY_SSL_MIN_PROTO_VERSION = [
    "default",
    "SSLv3",
    "TLSv1",
    "TLSv1-1",
    "TLSv1-2",
    "TLSv1-3",
]
VALID_BODY_AUTO_DISCOVER_KDC = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_user_exchange_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for user/exchange."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_user_exchange_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new user/exchange object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "connect-protocol" in payload:
        is_valid, error = _validate_enum_field(
            "connect-protocol",
            payload["connect-protocol"],
            VALID_BODY_CONNECT_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "validate-server-certificate" in payload:
        is_valid, error = _validate_enum_field(
            "validate-server-certificate",
            payload["validate-server-certificate"],
            VALID_BODY_VALIDATE_SERVER_CERTIFICATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-level" in payload:
        is_valid, error = _validate_enum_field(
            "auth-level",
            payload["auth-level"],
            VALID_BODY_AUTH_LEVEL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "http-auth-type",
            payload["http-auth-type"],
            VALID_BODY_HTTP_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auto-discover-kdc" in payload:
        is_valid, error = _validate_enum_field(
            "auto-discover-kdc",
            payload["auto-discover-kdc"],
            VALID_BODY_AUTO_DISCOVER_KDC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_user_exchange_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update user/exchange."""
    # Validate enum values using central function
    if "connect-protocol" in payload:
        is_valid, error = _validate_enum_field(
            "connect-protocol",
            payload["connect-protocol"],
            VALID_BODY_CONNECT_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "validate-server-certificate" in payload:
        is_valid, error = _validate_enum_field(
            "validate-server-certificate",
            payload["validate-server-certificate"],
            VALID_BODY_VALIDATE_SERVER_CERTIFICATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-level" in payload:
        is_valid, error = _validate_enum_field(
            "auth-level",
            payload["auth-level"],
            VALID_BODY_AUTH_LEVEL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "http-auth-type",
            payload["http-auth-type"],
            VALID_BODY_HTTP_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auto-discover-kdc" in payload:
        is_valid, error = _validate_enum_field(
            "auto-discover-kdc",
            payload["auto-discover-kdc"],
            VALID_BODY_AUTO_DISCOVER_KDC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "user/exchange",
    "category": "cmdb",
    "api_path": "user/exchange",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure MS Exchange server entries.",
    "total_fields": 14,
    "required_fields_count": 4,
    "fields_with_defaults_count": 12,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: icap/profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ProfileRespmodforwardrulesHeadergroupItem(TypedDict, total=False):
    """Nested item for respmod-forward-rules.header-group field."""
    id: int
    header_name: str
    header: str
    case_sensitivity: Literal["disable", "enable"]


class ProfileRespmodforwardrulesHttprespstatuscodeItem(TypedDict, total=False):
    """Nested item for respmod-forward-rules.http-resp-status-code field."""
    code: int


class ProfileIcapheadersItem(TypedDict, total=False):
    """Nested item for icap-headers field."""
    id: int
    name: str
    content: str
    base64_encoding: Literal["disable", "enable"]


class ProfileRespmodforwardrulesItem(TypedDict, total=False):
    """Nested item for respmod-forward-rules field."""
    name: str
    host: str
    header_group: str | list[str] | list[ProfileRespmodforwardrulesHeadergroupItem]
    action: Literal["forward", "bypass"]
    http_resp_status_code: str | list[str] | list[ProfileRespmodforwardrulesHttprespstatuscodeItem]


class ProfilePayload(TypedDict, total=False):
    """Payload type for Profile operations."""
    replacemsg_group: str
    name: str
    comment: str
    request: Literal["disable", "enable"]
    response: Literal["disable", "enable"]
    file_transfer: str | list[str]
    streaming_content_bypass: Literal["disable", "enable"]
    ocr_only: Literal["disable", "enable"]
    x204_size_limit: int
    x204_response: Literal["disable", "enable"]
    preview: Literal["disable", "enable"]
    preview_data_length: int
    request_server: str
    response_server: str
    file_transfer_server: str
    request_failure: Literal["error", "bypass"]
    response_failure: Literal["error", "bypass"]
    file_transfer_failure: Literal["error", "bypass"]
    request_path: str
    response_path: str
    file_transfer_path: str
    methods: str | list[str]
    response_req_hdr: Literal["disable", "enable"]
    respmod_default_action: Literal["forward", "bypass"]
    icap_block_log: Literal["disable", "enable"]
    chunk_encap: Literal["disable", "enable"]
    extension_feature: str | list[str]
    scan_progress_interval: int
    timeout: int
    icap_headers: str | list[str] | list[ProfileIcapheadersItem]
    respmod_forward_rules: str | list[str] | list[ProfileRespmodforwardrulesItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ProfileResponse(TypedDict, total=False):
    """Response type for Profile - use with .dict property for typed dict access."""
    replacemsg_group: str
    name: str
    comment: str
    request: Literal["disable", "enable"]
    response: Literal["disable", "enable"]
    file_transfer: str
    streaming_content_bypass: Literal["disable", "enable"]
    ocr_only: Literal["disable", "enable"]
    x204_size_limit: int
    x204_response: Literal["disable", "enable"]
    preview: Literal["disable", "enable"]
    preview_data_length: int
    request_server: str
    response_server: str
    file_transfer_server: str
    request_failure: Literal["error", "bypass"]
    response_failure: Literal["error", "bypass"]
    file_transfer_failure: Literal["error", "bypass"]
    request_path: str
    response_path: str
    file_transfer_path: str
    methods: str
    response_req_hdr: Literal["disable", "enable"]
    respmod_default_action: Literal["forward", "bypass"]
    icap_block_log: Literal["disable", "enable"]
    chunk_encap: Literal["disable", "enable"]
    extension_feature: str
    scan_progress_interval: int
    timeout: int
    icap_headers: list[ProfileIcapheadersItem]
    respmod_forward_rules: list[ProfileRespmodforwardrulesItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ProfileRespmodforwardrulesHeadergroupItemObject(FortiObject[ProfileRespmodforwardrulesHeadergroupItem]):
    """Typed object for respmod-forward-rules.header-group table items with attribute access."""
    id: int
    header_name: str
    header: str
    case_sensitivity: Literal["disable", "enable"]


class ProfileRespmodforwardrulesHttprespstatuscodeItemObject(FortiObject[ProfileRespmodforwardrulesHttprespstatuscodeItem]):
    """Typed object for respmod-forward-rules.http-resp-status-code table items with attribute access."""
    code: int


class ProfileIcapheadersItemObject(FortiObject[ProfileIcapheadersItem]):
    """Typed object for icap-headers table items with attribute access."""
    id: int
    name: str
    content: str
    base64_encoding: Literal["disable", "enable"]


class ProfileRespmodforwardrulesItemObject(FortiObject[ProfileRespmodforwardrulesItem]):
    """Typed object for respmod-forward-rules table items with attribute access."""
    name: str
    host: str
    header_group: FortiObjectList[ProfileRespmodforwardrulesHeadergroupItemObject]
    action: Literal["forward", "bypass"]
    http_resp_status_code: FortiObjectList[ProfileRespmodforwardrulesHttprespstatuscodeItemObject]


class ProfileObject(FortiObject):
    """Typed FortiObject for Profile with field access."""
    replacemsg_group: str
    name: str
    comment: str
    request: Literal["disable", "enable"]
    response: Literal["disable", "enable"]
    file_transfer: str
    streaming_content_bypass: Literal["disable", "enable"]
    ocr_only: Literal["disable", "enable"]
    x204_size_limit: int
    x204_response: Literal["disable", "enable"]
    preview: Literal["disable", "enable"]
    preview_data_length: int
    request_server: str
    response_server: str
    file_transfer_server: str
    request_failure: Literal["error", "bypass"]
    response_failure: Literal["error", "bypass"]
    file_transfer_failure: Literal["error", "bypass"]
    request_path: str
    response_path: str
    file_transfer_path: str
    methods: str
    response_req_hdr: Literal["disable", "enable"]
    respmod_default_action: Literal["forward", "bypass"]
    icap_block_log: Literal["disable", "enable"]
    chunk_encap: Literal["disable", "enable"]
    extension_feature: str
    scan_progress_interval: int
    timeout: int
    icap_headers: FortiObjectList[ProfileIcapheadersItemObject]
    respmod_forward_rules: FortiObjectList[ProfileRespmodforwardrulesItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class Profile:
    """
    
    Endpoint: icap/profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ProfilePayload | None = ...,
        replacemsg_group: str | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        request: Literal["disable", "enable"] | None = ...,
        response: Literal["disable", "enable"] | None = ...,
        file_transfer: str | list[str] | None = ...,
        streaming_content_bypass: Literal["disable", "enable"] | None = ...,
        ocr_only: Literal["disable", "enable"] | None = ...,
        x204_size_limit: int | None = ...,
        x204_response: Literal["disable", "enable"] | None = ...,
        preview: Literal["disable", "enable"] | None = ...,
        preview_data_length: int | None = ...,
        request_server: str | None = ...,
        response_server: str | None = ...,
        file_transfer_server: str | None = ...,
        request_failure: Literal["error", "bypass"] | None = ...,
        response_failure: Literal["error", "bypass"] | None = ...,
        file_transfer_failure: Literal["error", "bypass"] | None = ...,
        request_path: str | None = ...,
        response_path: str | None = ...,
        file_transfer_path: str | None = ...,
        methods: str | list[str] | None = ...,
        response_req_hdr: Literal["disable", "enable"] | None = ...,
        respmod_default_action: Literal["forward", "bypass"] | None = ...,
        icap_block_log: Literal["disable", "enable"] | None = ...,
        chunk_encap: Literal["disable", "enable"] | None = ...,
        extension_feature: str | list[str] | None = ...,
        scan_progress_interval: int | None = ...,
        timeout: int | None = ...,
        icap_headers: str | list[str] | list[ProfileIcapheadersItem] | None = ...,
        respmod_forward_rules: str | list[str] | list[ProfileRespmodforwardrulesItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ProfilePayload | None = ...,
        replacemsg_group: str | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        request: Literal["disable", "enable"] | None = ...,
        response: Literal["disable", "enable"] | None = ...,
        file_transfer: str | list[str] | None = ...,
        streaming_content_bypass: Literal["disable", "enable"] | None = ...,
        ocr_only: Literal["disable", "enable"] | None = ...,
        x204_size_limit: int | None = ...,
        x204_response: Literal["disable", "enable"] | None = ...,
        preview: Literal["disable", "enable"] | None = ...,
        preview_data_length: int | None = ...,
        request_server: str | None = ...,
        response_server: str | None = ...,
        file_transfer_server: str | None = ...,
        request_failure: Literal["error", "bypass"] | None = ...,
        response_failure: Literal["error", "bypass"] | None = ...,
        file_transfer_failure: Literal["error", "bypass"] | None = ...,
        request_path: str | None = ...,
        response_path: str | None = ...,
        file_transfer_path: str | None = ...,
        methods: str | list[str] | None = ...,
        response_req_hdr: Literal["disable", "enable"] | None = ...,
        respmod_default_action: Literal["forward", "bypass"] | None = ...,
        icap_block_log: Literal["disable", "enable"] | None = ...,
        chunk_encap: Literal["disable", "enable"] | None = ...,
        extension_feature: str | list[str] | None = ...,
        scan_progress_interval: int | None = ...,
        timeout: int | None = ...,
        icap_headers: str | list[str] | list[ProfileIcapheadersItem] | None = ...,
        respmod_forward_rules: str | list[str] | list[ProfileRespmodforwardrulesItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ProfilePayload | None = ...,
        replacemsg_group: str | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        request: Literal["disable", "enable"] | None = ...,
        response: Literal["disable", "enable"] | None = ...,
        file_transfer: Literal["ssh", "ftp"] | list[str] | None = ...,
        streaming_content_bypass: Literal["disable", "enable"] | None = ...,
        ocr_only: Literal["disable", "enable"] | None = ...,
        x204_size_limit: int | None = ...,
        x204_response: Literal["disable", "enable"] | None = ...,
        preview: Literal["disable", "enable"] | None = ...,
        preview_data_length: int | None = ...,
        request_server: str | None = ...,
        response_server: str | None = ...,
        file_transfer_server: str | None = ...,
        request_failure: Literal["error", "bypass"] | None = ...,
        response_failure: Literal["error", "bypass"] | None = ...,
        file_transfer_failure: Literal["error", "bypass"] | None = ...,
        request_path: str | None = ...,
        response_path: str | None = ...,
        file_transfer_path: str | None = ...,
        methods: Literal["delete", "get", "head", "options", "post", "put", "trace", "connect", "other"] | list[str] | None = ...,
        response_req_hdr: Literal["disable", "enable"] | None = ...,
        respmod_default_action: Literal["forward", "bypass"] | None = ...,
        icap_block_log: Literal["disable", "enable"] | None = ...,
        chunk_encap: Literal["disable", "enable"] | None = ...,
        extension_feature: Literal["scan-progress"] | list[str] | None = ...,
        scan_progress_interval: int | None = ...,
        timeout: int | None = ...,
        icap_headers: str | list[str] | list[ProfileIcapheadersItem] | None = ...,
        respmod_forward_rules: str | list[str] | list[ProfileRespmodforwardrulesItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Profile",
    "ProfilePayload",
    "ProfileResponse",
    "ProfileObject",
]


"""
Pydantic Models for CMDB - wireless_controller/timers

Runtime validation models for wireless_controller/timers configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class TimersModel(BaseModel):
    """
    Pydantic model for wireless_controller/timers configuration.
    
    Configure CAPWAP timers.
    
    Validation Rules:        - echo_interval: min=1 max=255 pattern=        - nat_session_keep_alive: min=0 max=255 pattern=        - discovery_interval: min=2 max=180 pattern=        - client_idle_timeout: min=20 max=3600 pattern=        - client_idle_rehome_timeout: min=2 max=3600 pattern=        - auth_timeout: min=5 max=30 pattern=        - rogue_ap_log: min=0 max=1440 pattern=        - fake_ap_log: min=1 max=1440 pattern=        - sta_offline_cleanup: min=0 max=4294967295 pattern=        - sta_offline_ip2mac_cleanup: min=0 max=4294967295 pattern=        - sta_cap_cleanup: min=0 max=4294967295 pattern=        - rogue_ap_cleanup: min=0 max=4294967295 pattern=        - rogue_sta_cleanup: min=0 max=4294967295 pattern=        - wids_entry_cleanup: min=0 max=4294967295 pattern=        - ble_device_cleanup: min=0 max=4294967295 pattern=        - sta_stats_interval: min=1 max=255 pattern=        - vap_stats_interval: min=1 max=255 pattern=        - radio_stats_interval: min=1 max=255 pattern=        - sta_capability_interval: min=1 max=255 pattern=        - sta_locate_timer: min=0 max=86400 pattern=        - ipsec_intf_cleanup: min=30 max=3600 pattern=        - ble_scan_report_intv: min=10 max=3600 pattern=        - drma_interval: min=1 max=1440 pattern=        - ap_reboot_wait_interval1: min=5 max=65535 pattern=        - ap_reboot_wait_time: max_length=7 pattern=        - ap_reboot_wait_interval2: min=5 max=65535 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    echo_interval: int | None = Field(ge=1, le=255, default=30, description="Time between echo requests sent by the managed WTP, AP, or FortiAP (1 - 255 sec, default = 30).")    
    nat_session_keep_alive: int | None = Field(ge=0, le=255, default=0, description="Maximal time in seconds between control requests sent by the managed WTP, AP, or FortiAP (0 - 255 sec, default = 0).")    
    discovery_interval: int | None = Field(ge=2, le=180, default=5, description="Time between discovery requests (2 - 180 sec, default = 5).")    
    client_idle_timeout: int | None = Field(ge=20, le=3600, default=300, description="Time after which a client is considered idle and times out (20 - 3600 sec, default = 300, 0 for no timeout).")    
    client_idle_rehome_timeout: int | None = Field(ge=2, le=3600, default=20, description="Time after which a client is considered idle and disconnected from the home controller (2 - 3600 sec, default = 20, 0 for no timeout).")    
    auth_timeout: int | None = Field(ge=5, le=30, default=5, description="Time after which a client is considered failed in RADIUS authentication and times out (5 - 30 sec, default = 5).")    
    rogue_ap_log: int | None = Field(ge=0, le=1440, default=0, description="Time between logging rogue AP messages if periodic rogue AP logging is configured (0 - 1440 min, default = 0).")    
    fake_ap_log: int | None = Field(ge=1, le=1440, default=1, description="Time between recording logs about fake APs if periodic fake AP logging is configured (1 - 1440 min, default = 1).")    
    sta_offline_cleanup: int | None = Field(ge=0, le=4294967295, default=300, description="Time period in seconds to keep station offline data after it is gone (default = 300).")    
    sta_offline_ip2mac_cleanup: int | None = Field(ge=0, le=4294967295, default=300, description="Time period in seconds to keep station offline Ip2mac data after it is gone (default = 300).")    
    sta_cap_cleanup: int | None = Field(ge=0, le=4294967295, default=0, description="Time period in minutes to keep station capability data after it is gone (default = 0).")    
    rogue_ap_cleanup: int | None = Field(ge=0, le=4294967295, default=0, description="Time period in minutes to keep rogue AP after it is gone (default = 0).")    
    rogue_sta_cleanup: int | None = Field(ge=0, le=4294967295, default=0, description="Time period in minutes to keep rogue station after it is gone (default = 0).")    
    wids_entry_cleanup: int | None = Field(ge=0, le=4294967295, default=0, description="Time period in minutes to keep wids entry after it is gone (default = 0).")    
    ble_device_cleanup: int | None = Field(ge=0, le=4294967295, default=60, description="Time period in minutes to keep BLE device after it is gone (default = 60).")    
    sta_stats_interval: int | None = Field(ge=1, le=255, default=10, description="Time between running client (station) reports (1 - 255 sec, default = 10).")    
    vap_stats_interval: int | None = Field(ge=1, le=255, default=15, description="Time between running Virtual Access Point (VAP) reports (1 - 255 sec, default = 15).")    
    radio_stats_interval: int | None = Field(ge=1, le=255, default=15, description="Time between running radio reports (1 - 255 sec, default = 15).")    
    sta_capability_interval: int | None = Field(ge=1, le=255, default=30, description="Time between running station capability reports (1 - 255 sec, default = 30).")    
    sta_locate_timer: int | None = Field(ge=0, le=86400, default=1800, description="Time between running client presence flushes to remove clients that are listed but no longer present (0 - 86400 sec, default = 1800).")    
    ipsec_intf_cleanup: int | None = Field(ge=30, le=3600, default=120, description="Time period to keep IPsec VPN interfaces up after WTP sessions are disconnected (30 - 3600 sec, default = 120).")    
    ble_scan_report_intv: int | None = Field(ge=10, le=3600, default=30, description="Time between running Bluetooth Low Energy (BLE) reports (10 - 3600 sec, default = 30).")    
    drma_interval: int | None = Field(ge=1, le=1440, default=60, description="Dynamic radio mode assignment (DRMA) schedule interval in minutes (1 - 1440, default = 60).")    
    ap_reboot_wait_interval1: int | None = Field(ge=5, le=65535, default=0, description="Time in minutes to wait before AP reboots when there is no controller detected (5 - 65535, default = 0, 0 for no reboot).")    
    ap_reboot_wait_time: str | None = Field(max_length=7, default=None, description="Time to reboot the AP when there is no controller detected and standalone SSIDs are pushed to the AP in the previous session, format hh:mm.")    
    ap_reboot_wait_interval2: int | None = Field(ge=5, le=65535, default=0, description="Time in minutes to wait before AP reboots when there is no controller detected and standalone SSIDs are pushed to the AP in the previous session (5 - 65535, default = 0, 0 for no reboot).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "TimersModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "TimersModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.060438Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/alarm
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class AlarmGroupsFwpolicyviolationsItem(TypedDict, total=False):
    """Nested item for groups.fw-policy-violations field."""
    id: int
    threshold: int
    src_ip: str
    dst_ip: str
    src_port: int
    dst_port: int


class AlarmGroupsItem(TypedDict, total=False):
    """Nested item for groups field."""
    id: int
    period: int
    admin_auth_failure_threshold: int
    admin_auth_lockout_threshold: int
    user_auth_failure_threshold: int
    user_auth_lockout_threshold: int
    replay_attempt_threshold: int
    self_test_failure_threshold: int
    log_full_warning_threshold: int
    encryption_failure_threshold: int
    decryption_failure_threshold: int
    fw_policy_violations: str | list[str] | list[AlarmGroupsFwpolicyviolationsItem]
    fw_policy_id: int
    fw_policy_id_threshold: int


class AlarmPayload(TypedDict, total=False):
    """Payload type for Alarm operations."""
    status: Literal["enable", "disable"]
    audible: Literal["enable", "disable"]
    groups: str | list[str] | list[AlarmGroupsItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class AlarmResponse(TypedDict, total=False):
    """Response type for Alarm - use with .dict property for typed dict access."""
    status: Literal["enable", "disable"]
    audible: Literal["enable", "disable"]
    groups: list[AlarmGroupsItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class AlarmGroupsFwpolicyviolationsItemObject(FortiObject[AlarmGroupsFwpolicyviolationsItem]):
    """Typed object for groups.fw-policy-violations table items with attribute access."""
    id: int
    threshold: int
    src_ip: str
    dst_ip: str
    src_port: int
    dst_port: int


class AlarmGroupsItemObject(FortiObject[AlarmGroupsItem]):
    """Typed object for groups table items with attribute access."""
    id: int
    period: int
    admin_auth_failure_threshold: int
    admin_auth_lockout_threshold: int
    user_auth_failure_threshold: int
    user_auth_lockout_threshold: int
    replay_attempt_threshold: int
    self_test_failure_threshold: int
    log_full_warning_threshold: int
    encryption_failure_threshold: int
    decryption_failure_threshold: int
    fw_policy_violations: FortiObjectList[AlarmGroupsFwpolicyviolationsItemObject]
    fw_policy_id: int
    fw_policy_id_threshold: int


class AlarmObject(FortiObject):
    """Typed FortiObject for Alarm with field access."""
    status: Literal["enable", "disable"]
    audible: Literal["enable", "disable"]
    groups: FortiObjectList[AlarmGroupsItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class Alarm:
    """
    
    Endpoint: system/alarm
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> AlarmObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: AlarmPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        audible: Literal["enable", "disable"] | None = ...,
        groups: str | list[str] | list[AlarmGroupsItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> AlarmObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: AlarmPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        audible: Literal["enable", "disable"] | None = ...,
        groups: str | list[str] | list[AlarmGroupsItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Alarm",
    "AlarmPayload",
    "AlarmResponse",
    "AlarmObject",
]


"""Validation helpers for system/automation_trigger - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "stitch-name",  # Triggering stitch name.
    "faz-event-name",  # FortiAnalyzer event handler name.
    "serial",  # Fabric connector serial number.
    "fabric-event-name",  # Fabric connector event handler name.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "trigger-type": "event-based",
    "event-type": "ioc",
    "license-type": "forticare-support",
    "report-type": "posture",
    "stitch-name": "",
    "trigger-frequency": "daily",
    "trigger-weekday": "",
    "trigger-day": 1,
    "trigger-hour": 0,
    "trigger-minute": 0,
    "trigger-datetime": "0000-00-00 00:00:00",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Name.
    "description": "var-string",  # Description.
    "trigger-type": "option",  # Trigger type.
    "event-type": "option",  # Event type.
    "vdom": "string",  # Virtual domain(s) that this trigger is valid for.
    "license-type": "option",  # License type.
    "report-type": "option",  # Security Rating report.
    "stitch-name": "string",  # Triggering stitch name.
    "logid": "string",  # Log IDs to trigger event.
    "trigger-frequency": "option",  # Scheduled trigger frequency (default = daily).
    "trigger-weekday": "option",  # Day of week for trigger.
    "trigger-day": "integer",  # Day within a month to trigger.
    "trigger-hour": "integer",  # Hour of the day on which to trigger (0 - 23, default = 1).
    "trigger-minute": "integer",  # Minute of the hour on which to trigger (0 - 59, default = 0)
    "trigger-datetime": "datetime",  # Trigger date and time (YYYY-MM-DD HH:MM:SS).
    "fields": "string",  # Customized trigger field settings.
    "faz-event-name": "var-string",  # FortiAnalyzer event handler name.
    "faz-event-severity": "var-string",  # FortiAnalyzer event severity.
    "faz-event-tags": "var-string",  # FortiAnalyzer event tags.
    "serial": "var-string",  # Fabric connector serial number.
    "fabric-event-name": "var-string",  # Fabric connector event handler name.
    "fabric-event-severity": "var-string",  # Fabric connector event severity.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Name.",
    "description": "Description.",
    "trigger-type": "Trigger type.",
    "event-type": "Event type.",
    "vdom": "Virtual domain(s) that this trigger is valid for.",
    "license-type": "License type.",
    "report-type": "Security Rating report.",
    "stitch-name": "Triggering stitch name.",
    "logid": "Log IDs to trigger event.",
    "trigger-frequency": "Scheduled trigger frequency (default = daily).",
    "trigger-weekday": "Day of week for trigger.",
    "trigger-day": "Day within a month to trigger.",
    "trigger-hour": "Hour of the day on which to trigger (0 - 23, default = 1).",
    "trigger-minute": "Minute of the hour on which to trigger (0 - 59, default = 0).",
    "trigger-datetime": "Trigger date and time (YYYY-MM-DD HH:MM:SS).",
    "fields": "Customized trigger field settings.",
    "faz-event-name": "FortiAnalyzer event handler name.",
    "faz-event-severity": "FortiAnalyzer event severity.",
    "faz-event-tags": "FortiAnalyzer event tags.",
    "serial": "Fabric connector serial number.",
    "fabric-event-name": "Fabric connector event handler name.",
    "fabric-event-severity": "Fabric connector event severity.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "stitch-name": {"type": "string", "max_length": 35},
    "trigger-day": {"type": "integer", "min": 1, "max": 31},
    "trigger-hour": {"type": "integer", "min": 0, "max": 23},
    "trigger-minute": {"type": "integer", "min": 0, "max": 59},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "vdom": {
        "name": {
            "type": "string",
            "help": "Virtual domain name.",
            "default": "",
            "max_length": 79,
        },
    },
    "logid": {
        "id": {
            "type": "integer",
            "help": "Log ID.",
            "required": True,
            "default": 0,
            "min_value": 1,
            "max_value": 65535,
        },
    },
    "fields": {
        "id": {
            "type": "integer",
            "help": "Entry ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "name": {
            "type": "string",
            "help": "Name.",
            "default": "",
            "max_length": 35,
        },
        "value": {
            "type": "var-string",
            "help": "Value.",
            "max_length": 63,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_TRIGGER_TYPE = [
    "event-based",
    "scheduled",
]
VALID_BODY_EVENT_TYPE = [
    "ioc",
    "event-log",
    "reboot",
    "low-memory",
    "high-cpu",
    "license-near-expiry",
    "local-cert-near-expiry",
    "ha-failover",
    "config-change",
    "security-rating-summary",
    "virus-ips-db-updated",
    "faz-event",
    "incoming-webhook",
    "fabric-event",
    "ips-logs",
    "anomaly-logs",
    "virus-logs",
    "ssh-logs",
    "webfilter-violation",
    "traffic-violation",
    "stitch",
]
VALID_BODY_LICENSE_TYPE = [
    "forticare-support",
    "fortiguard-webfilter",
    "fortiguard-antispam",
    "fortiguard-antivirus",
    "fortiguard-ips",
    "fortiguard-management",
    "forticloud",
    "any",
]
VALID_BODY_REPORT_TYPE = [
    "posture",
    "coverage",
    "optimization",
    "any",
]
VALID_BODY_TRIGGER_FREQUENCY = [
    "hourly",
    "daily",
    "weekly",
    "monthly",
    "once",
]
VALID_BODY_TRIGGER_WEEKDAY = [
    "sunday",
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday",
    "saturday",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_automation_trigger_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/automation_trigger."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_automation_trigger_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/automation_trigger object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "trigger-type" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-type",
            payload["trigger-type"],
            VALID_BODY_TRIGGER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "event-type" in payload:
        is_valid, error = _validate_enum_field(
            "event-type",
            payload["event-type"],
            VALID_BODY_EVENT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "license-type" in payload:
        is_valid, error = _validate_enum_field(
            "license-type",
            payload["license-type"],
            VALID_BODY_LICENSE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "report-type" in payload:
        is_valid, error = _validate_enum_field(
            "report-type",
            payload["report-type"],
            VALID_BODY_REPORT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "trigger-frequency" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-frequency",
            payload["trigger-frequency"],
            VALID_BODY_TRIGGER_FREQUENCY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "trigger-weekday" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-weekday",
            payload["trigger-weekday"],
            VALID_BODY_TRIGGER_WEEKDAY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_automation_trigger_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/automation_trigger."""
    # Validate enum values using central function
    if "trigger-type" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-type",
            payload["trigger-type"],
            VALID_BODY_TRIGGER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "event-type" in payload:
        is_valid, error = _validate_enum_field(
            "event-type",
            payload["event-type"],
            VALID_BODY_EVENT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "license-type" in payload:
        is_valid, error = _validate_enum_field(
            "license-type",
            payload["license-type"],
            VALID_BODY_LICENSE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "report-type" in payload:
        is_valid, error = _validate_enum_field(
            "report-type",
            payload["report-type"],
            VALID_BODY_REPORT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "trigger-frequency" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-frequency",
            payload["trigger-frequency"],
            VALID_BODY_TRIGGER_FREQUENCY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "trigger-weekday" in payload:
        is_valid, error = _validate_enum_field(
            "trigger-weekday",
            payload["trigger-weekday"],
            VALID_BODY_TRIGGER_WEEKDAY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/automation_trigger",
    "category": "cmdb",
    "api_path": "system/automation-trigger",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Trigger for automation stitches.",
    "total_fields": 22,
    "required_fields_count": 4,
    "fields_with_defaults_count": 12,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: user/domain_controller
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class DomainControllerExtraserverItem(TypedDict, total=False):
    """Nested item for extra-server field."""
    id: int
    ip_address: str
    port: int
    source_ip_address: str
    source_port: int


class DomainControllerLdapserverItem(TypedDict, total=False):
    """Nested item for ldap-server field."""
    name: str


class DomainControllerPayload(TypedDict, total=False):
    """Payload type for DomainController operations."""
    name: str
    ad_mode: Literal["none", "ds", "lds"]
    hostname: str
    username: str
    password: str
    ip_address: str
    ip6: str
    port: int
    source_ip_address: str
    source_ip6: str
    source_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    extra_server: str | list[str] | list[DomainControllerExtraserverItem]
    domain_name: str
    replication_port: int
    ldap_server: str | list[str] | list[DomainControllerLdapserverItem]
    change_detection: Literal["enable", "disable"]
    change_detection_period: int
    dns_srv_lookup: Literal["enable", "disable"]
    adlds_dn: str
    adlds_ip_address: str
    adlds_ip6: str
    adlds_port: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class DomainControllerResponse(TypedDict, total=False):
    """Response type for DomainController - use with .dict property for typed dict access."""
    name: str
    ad_mode: Literal["none", "ds", "lds"]
    hostname: str
    username: str
    password: str
    ip_address: str
    ip6: str
    port: int
    source_ip_address: str
    source_ip6: str
    source_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    extra_server: list[DomainControllerExtraserverItem]
    domain_name: str
    replication_port: int
    ldap_server: list[DomainControllerLdapserverItem]
    change_detection: Literal["enable", "disable"]
    change_detection_period: int
    dns_srv_lookup: Literal["enable", "disable"]
    adlds_dn: str
    adlds_ip_address: str
    adlds_ip6: str
    adlds_port: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class DomainControllerExtraserverItemObject(FortiObject[DomainControllerExtraserverItem]):
    """Typed object for extra-server table items with attribute access."""
    id: int
    ip_address: str
    port: int
    source_ip_address: str
    source_port: int


class DomainControllerLdapserverItemObject(FortiObject[DomainControllerLdapserverItem]):
    """Typed object for ldap-server table items with attribute access."""
    name: str


class DomainControllerObject(FortiObject):
    """Typed FortiObject for DomainController with field access."""
    name: str
    ad_mode: Literal["none", "ds", "lds"]
    hostname: str
    username: str
    password: str
    ip_address: str
    ip6: str
    port: int
    source_ip_address: str
    source_ip6: str
    source_port: int
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    extra_server: FortiObjectList[DomainControllerExtraserverItemObject]
    domain_name: str
    replication_port: int
    ldap_server: FortiObjectList[DomainControllerLdapserverItemObject]
    change_detection: Literal["enable", "disable"]
    change_detection_period: int
    dns_srv_lookup: Literal["enable", "disable"]
    adlds_dn: str
    adlds_ip_address: str
    adlds_ip6: str
    adlds_port: int


# ================================================================
# Main Endpoint Class
# ================================================================

class DomainController:
    """
    
    Endpoint: user/domain_controller
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DomainControllerObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[DomainControllerObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: DomainControllerPayload | None = ...,
        name: str | None = ...,
        ad_mode: Literal["none", "ds", "lds"] | None = ...,
        hostname: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip_address: str | None = ...,
        ip6: str | None = ...,
        port: int | None = ...,
        source_ip_address: str | None = ...,
        source_ip6: str | None = ...,
        source_port: int | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        extra_server: str | list[str] | list[DomainControllerExtraserverItem] | None = ...,
        domain_name: str | None = ...,
        replication_port: int | None = ...,
        ldap_server: str | list[str] | list[DomainControllerLdapserverItem] | None = ...,
        change_detection: Literal["enable", "disable"] | None = ...,
        change_detection_period: int | None = ...,
        dns_srv_lookup: Literal["enable", "disable"] | None = ...,
        adlds_dn: str | None = ...,
        adlds_ip_address: str | None = ...,
        adlds_ip6: str | None = ...,
        adlds_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DomainControllerObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: DomainControllerPayload | None = ...,
        name: str | None = ...,
        ad_mode: Literal["none", "ds", "lds"] | None = ...,
        hostname: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip_address: str | None = ...,
        ip6: str | None = ...,
        port: int | None = ...,
        source_ip_address: str | None = ...,
        source_ip6: str | None = ...,
        source_port: int | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        extra_server: str | list[str] | list[DomainControllerExtraserverItem] | None = ...,
        domain_name: str | None = ...,
        replication_port: int | None = ...,
        ldap_server: str | list[str] | list[DomainControllerLdapserverItem] | None = ...,
        change_detection: Literal["enable", "disable"] | None = ...,
        change_detection_period: int | None = ...,
        dns_srv_lookup: Literal["enable", "disable"] | None = ...,
        adlds_dn: str | None = ...,
        adlds_ip_address: str | None = ...,
        adlds_ip6: str | None = ...,
        adlds_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DomainControllerObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: DomainControllerPayload | None = ...,
        name: str | None = ...,
        ad_mode: Literal["none", "ds", "lds"] | None = ...,
        hostname: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip_address: str | None = ...,
        ip6: str | None = ...,
        port: int | None = ...,
        source_ip_address: str | None = ...,
        source_ip6: str | None = ...,
        source_port: int | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        extra_server: str | list[str] | list[DomainControllerExtraserverItem] | None = ...,
        domain_name: str | None = ...,
        replication_port: int | None = ...,
        ldap_server: str | list[str] | list[DomainControllerLdapserverItem] | None = ...,
        change_detection: Literal["enable", "disable"] | None = ...,
        change_detection_period: int | None = ...,
        dns_srv_lookup: Literal["enable", "disable"] | None = ...,
        adlds_dn: str | None = ...,
        adlds_ip_address: str | None = ...,
        adlds_ip6: str | None = ...,
        adlds_port: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "DomainController",
    "DomainControllerPayload",
    "DomainControllerResponse",
    "DomainControllerObject",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: router/multicast
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class MulticastPimsmglobalvrfRpaddressItem(TypedDict, total=False):
    """Nested item for pim-sm-global-vrf.rp-address field."""
    id: int
    ip_address: str
    group: str


class MulticastInterfaceJoingroupItem(TypedDict, total=False):
    """Nested item for interface.join-group field."""
    address: str


class MulticastInterfaceIgmpDict(TypedDict, total=False):
    """Nested object type for interface.igmp field."""
    access_group: str
    version: Literal["3", "2", "1"]
    immediate_leave_group: str
    last_member_query_interval: int
    last_member_query_count: int
    query_max_response_time: int
    query_interval: int
    query_timeout: int
    router_alert_check: Literal["enable", "disable"]


class MulticastPimsmglobalRpaddressItem(TypedDict, total=False):
    """Nested item for pim-sm-global.rp-address field."""
    id: int
    ip_address: str
    group: str


class MulticastPimsmglobalDict(TypedDict, total=False):
    """Nested object type for pim-sm-global field."""
    message_interval: int
    join_prune_holdtime: int
    accept_register_list: str
    accept_source_list: str
    bsr_candidate: Literal["enable", "disable"]
    bsr_interface: str
    bsr_priority: int
    bsr_hash: int
    bsr_allow_quick_refresh: Literal["enable", "disable"]
    cisco_crp_prefix: Literal["enable", "disable"]
    cisco_register_checksum: Literal["enable", "disable"]
    cisco_register_checksum_group: str
    cisco_ignore_rp_set_priority: Literal["enable", "disable"]
    register_rp_reachability: Literal["enable", "disable"]
    register_source: Literal["disable", "interface", "ip-address"]
    register_source_interface: str
    register_source_ip: str
    register_supression: int
    null_register_retries: int
    rp_register_keepalive: int
    spt_threshold: Literal["enable", "disable"]
    spt_threshold_group: str
    ssm: Literal["enable", "disable"]
    ssm_range: str
    register_rate_limit: int
    pim_use_sdwan: Literal["enable", "disable"]
    rp_address: str | list[str] | list[MulticastPimsmglobalRpaddressItem]


class MulticastPimsmglobalvrfItem(TypedDict, total=False):
    """Nested item for pim-sm-global-vrf field."""
    vrf: int
    bsr_candidate: Literal["enable", "disable"]
    bsr_interface: str
    bsr_priority: int
    bsr_hash: int
    bsr_allow_quick_refresh: Literal["enable", "disable"]
    cisco_crp_prefix: Literal["enable", "disable"]
    rp_address: str | list[str] | list[MulticastPimsmglobalvrfRpaddressItem]


class MulticastInterfaceItem(TypedDict, total=False):
    """Nested item for interface field."""
    name: str
    ttl_threshold: int
    pim_mode: Literal["sparse-mode", "dense-mode"]
    passive: Literal["enable", "disable"]
    bfd: Literal["enable", "disable"]
    neighbour_filter: str
    hello_interval: int
    hello_holdtime: int
    cisco_exclude_genid: Literal["enable", "disable"]
    dr_priority: int
    propagation_delay: int
    state_refresh_interval: int
    rp_candidate: Literal["enable", "disable"]
    rp_candidate_group: str
    rp_candidate_priority: int
    rp_candidate_interval: int
    multicast_flow: str
    static_group: str
    rpf_nbr_fail_back: Literal["enable", "disable"]
    rpf_nbr_fail_back_filter: str
    join_group: str | list[str] | list[MulticastInterfaceJoingroupItem]
    igmp: MulticastInterfaceIgmpDict


class MulticastPayload(TypedDict, total=False):
    """Payload type for Multicast operations."""
    route_threshold: int
    route_limit: int
    multicast_routing: Literal["enable", "disable"]
    pim_sm_global: MulticastPimsmglobalDict
    pim_sm_global_vrf: str | list[str] | list[MulticastPimsmglobalvrfItem]
    interface: str | list[str] | list[MulticastInterfaceItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class MulticastResponse(TypedDict, total=False):
    """Response type for Multicast - use with .dict property for typed dict access."""
    route_threshold: int
    route_limit: int
    multicast_routing: Literal["enable", "disable"]
    pim_sm_global: MulticastPimsmglobalDict
    pim_sm_global_vrf: list[MulticastPimsmglobalvrfItem]
    interface: list[MulticastInterfaceItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class MulticastPimsmglobalvrfRpaddressItemObject(FortiObject[MulticastPimsmglobalvrfRpaddressItem]):
    """Typed object for pim-sm-global-vrf.rp-address table items with attribute access."""
    id: int
    ip_address: str
    group: str


class MulticastInterfaceJoingroupItemObject(FortiObject[MulticastInterfaceJoingroupItem]):
    """Typed object for interface.join-group table items with attribute access."""
    address: str


class MulticastPimsmglobalvrfItemObject(FortiObject[MulticastPimsmglobalvrfItem]):
    """Typed object for pim-sm-global-vrf table items with attribute access."""
    vrf: int
    bsr_candidate: Literal["enable", "disable"]
    bsr_interface: str
    bsr_priority: int
    bsr_hash: int
    bsr_allow_quick_refresh: Literal["enable", "disable"]
    cisco_crp_prefix: Literal["enable", "disable"]
    rp_address: FortiObjectList[MulticastPimsmglobalvrfRpaddressItemObject]


class MulticastInterfaceItemObject(FortiObject[MulticastInterfaceItem]):
    """Typed object for interface table items with attribute access."""
    name: str
    ttl_threshold: int
    pim_mode: Literal["sparse-mode", "dense-mode"]
    passive: Literal["enable", "disable"]
    bfd: Literal["enable", "disable"]
    neighbour_filter: str
    hello_interval: int
    hello_holdtime: int
    cisco_exclude_genid: Literal["enable", "disable"]
    dr_priority: int
    propagation_delay: int
    state_refresh_interval: int
    rp_candidate: Literal["enable", "disable"]
    rp_candidate_group: str
    rp_candidate_priority: int
    rp_candidate_interval: int
    multicast_flow: str
    static_group: str
    rpf_nbr_fail_back: Literal["enable", "disable"]
    rpf_nbr_fail_back_filter: str
    join_group: FortiObjectList[MulticastInterfaceJoingroupItemObject]
    igmp: MulticastInterfaceIgmpObject


class MulticastInterfaceIgmpObject(FortiObject):
    """Nested object for interface.igmp field with attribute access."""
    access_group: str
    version: Literal["3", "2", "1"]
    immediate_leave_group: str
    last_member_query_interval: int
    last_member_query_count: int
    query_max_response_time: int
    query_interval: int
    query_timeout: int
    router_alert_check: Literal["enable", "disable"]


class MulticastPimsmglobalRpaddressItemObject(FortiObject[MulticastPimsmglobalRpaddressItem]):
    """Typed object for pim-sm-global.rp-address table items with attribute access."""
    id: int
    ip_address: str
    group: str


class MulticastPimsmglobalObject(FortiObject):
    """Nested object for pim-sm-global field with attribute access."""
    message_interval: int
    join_prune_holdtime: int
    accept_register_list: str
    accept_source_list: str
    bsr_candidate: Literal["enable", "disable"]
    bsr_interface: str
    bsr_priority: int
    bsr_hash: int
    bsr_allow_quick_refresh: Literal["enable", "disable"]
    cisco_crp_prefix: Literal["enable", "disable"]
    cisco_register_checksum: Literal["enable", "disable"]
    cisco_register_checksum_group: str
    cisco_ignore_rp_set_priority: Literal["enable", "disable"]
    register_rp_reachability: Literal["enable", "disable"]
    register_source: Literal["disable", "interface", "ip-address"]
    register_source_interface: str
    register_source_ip: str
    register_supression: int
    null_register_retries: int
    rp_register_keepalive: int
    spt_threshold: Literal["enable", "disable"]
    spt_threshold_group: str
    ssm: Literal["enable", "disable"]
    ssm_range: str
    register_rate_limit: int
    pim_use_sdwan: Literal["enable", "disable"]
    rp_address: str | list[str]


class MulticastObject(FortiObject):
    """Typed FortiObject for Multicast with field access."""
    route_threshold: int
    route_limit: int
    multicast_routing: Literal["enable", "disable"]
    pim_sm_global: MulticastPimsmglobalObject
    pim_sm_global_vrf: FortiObjectList[MulticastPimsmglobalvrfItemObject]
    interface: FortiObjectList[MulticastInterfaceItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class Multicast:
    """
    
    Endpoint: router/multicast
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> MulticastObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: MulticastPayload | None = ...,
        route_threshold: int | None = ...,
        route_limit: int | None = ...,
        multicast_routing: Literal["enable", "disable"] | None = ...,
        pim_sm_global: MulticastPimsmglobalDict | None = ...,
        pim_sm_global_vrf: str | list[str] | list[MulticastPimsmglobalvrfItem] | None = ...,
        interface: str | list[str] | list[MulticastInterfaceItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> MulticastObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: MulticastPayload | None = ...,
        route_threshold: int | None = ...,
        route_limit: int | None = ...,
        multicast_routing: Literal["enable", "disable"] | None = ...,
        pim_sm_global: MulticastPimsmglobalDict | None = ...,
        pim_sm_global_vrf: str | list[str] | list[MulticastPimsmglobalvrfItem] | None = ...,
        interface: str | list[str] | list[MulticastInterfaceItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Multicast",
    "MulticastPayload",
    "MulticastResponse",
    "MulticastObject",
]